@@ -42,12 +42,14 @@
 #include "dvbstream.h"
 #include "dvbevents.h"
 #include "gdvb.h"
+#include "dvbcam.h"
+#include "kaffeinedvbplugin.h"
 
 #define MAXTUNETRY 1
 
 
 
-DvbStream::DvbStream( int anum, int tnum, fe_type_t type, const QString &src, const QString &charset )
+DvbStream::DvbStream( int anum, int tnum, fe_type_t type, const QString &src, bool as, const QString &charset )
 {
 	isRunning = false;
 	timeShifting = false;
@@ -63,17 +65,40 @@ DvbStream::DvbStream( int anum, int tnum, fe_type_t type, const QString &src, co
 	dvrName = QString("/dev/dvb/adapter%1/dvr%2").arg( adapter ).arg( tuner );
 	demuxName = QString("/dev/dvb/adapter%1/demux%2").arg( adapter ).arg( tuner );
 	dvbType = type;
+	canAutoscan = as,
 
 	out.setAutoDelete( true );
 
 	bool bok;
 	dvbEvents = new DVBevents( &bok, adapter, tuner, charset );
 
+	camProbed = false;
+	cam = NULL;
+
 	connect( &statusTimer, SIGNAL(timeout()), this, SLOT(checkStatus()) );
 }
 
 
 
+void DvbStream::setPlug( KaffeineDvbPlugin *p )
+{
+	plug = p;
+}
+
+
+
+void DvbStream::probeCam()
+{
+	int ci_type=0;
+	if ( camProbed )
+		return;
+	if ( (ci_type=DvbCam::probe( adapter, 0 ))>0 )
+		cam = new DvbCam( adapter, 0, tuner, ci_type );
+	camProbed = true;
+}
+
+
+
 void DvbStream::setLnb( int pos, LNB b )
 {
 	lnb[pos] = b;
@@ -99,8 +124,16 @@ void DvbStream::setSources( const QString &src )
 
 
 
-QStringList DvbStream::getSources()
+QStringList DvbStream::getSources( bool all )
 {
+	if ( !all ) {
+		if ( dvbType==FE_OFDM )
+			return QStringList( "Terrestrial" );
+		if ( dvbType==FE_QAM )
+			return QStringList( "Cable" );
+		if ( dvbType==FE_ATSC )
+			return QStringList( "Atsc" );
+	}
 	return source;
 }
 
@@ -111,6 +144,12 @@ bool DvbStream::canSource( ChannelDesc *chan )
 	if ( chan->tp.type!=dvbType ) {
 		return false;
 	}
+	if ( dvbType!=FE_QPSK ) {
+		if ( chan->tp.type==dvbType )
+			return true;
+		else
+			return false;
+	}
 	int i = getSatPos( chan->tp.source );
 	if ( i!=-1 )
 		return true;
@@ -281,6 +320,8 @@ bool DvbStream::tuneDvb( ChannelDesc *chan, bool dvr )
 		return false;
 	}
 
+	probeCam();
+
 	fprintf(stderr, "Using DVB device %d:%d \"%s\"\n", adapter, tuner, fe_info.name);
 
 	freq*=1000;
@@ -346,6 +387,13 @@ bool DvbStream::tuneDvb( ChannelDesc *chan, bool dvr )
 			}
 			break;
 		}
+		case FE_ATSC : {
+			fprintf(stderr,"tuning ATSC to %lu\n", freq);
+			feparams.frequency=freq;
+			feparams.u.vsb.modulation = chan->tp.modulation;
+			fprintf(stderr,"inv:%d mod:%d\n", chan->tp.inversion, chan->tp.modulation );
+			break;
+		}
 	}
 	if ( chan->tp.inversion==INVERSION_AUTO ) {
 		if ( fe_info.caps & FE_CAN_INVERSION_AUTO )
@@ -364,7 +412,7 @@ bool DvbStream::tuneDvb( ChannelDesc *chan, bool dvr )
 		return false;
 	}
 
-	for ( int i=0; i<30; i++ ) {
+	for ( int i=0; i<15; i++ ) {
 		usleep( 100000 );
 		fprintf( stderr, "." );
 		if ( ioctl( fdFrontend, FE_READ_STATUS, &status )==-1 ) {
@@ -569,6 +617,15 @@ bool DvbStream::checkStatus()
 
 
 
+int DvbStream::getSNR()
+{
+	int32_t snr;
+	ioctl( fdFrontend, FE_READ_SNR, &snr );
+	return snr*100/65535;
+}
+
+
+
 bool DvbStream::hasVideo()
 {
 	if ( getLiveChannel().vpid )
@@ -673,18 +730,38 @@ bool DvbStream::running() const
 
 void DvbStream::receivePlayDvb()
 {
-	//KApplication::kApplication()->postEvent( videoWin, new QTimerEvent( 500 ) );
+	KApplication::kApplication()->postEvent( this, new QTimerEvent( 500 ) );
+}
+
+void DvbStream::timerEvent( QTimerEvent* ev )
+{
+	switch ( ev->timerId() ) {
+		case 500:
+			emit playDvb();
+	}
 }
 
 
 
 void DvbStream::recordEnded( DVBout *o, RecTimer* t, bool kill )
 {
+	int i;
+
 	if ( kill ) {
 		removePids( o );
 		removeOut( o );
 		if ( out.count()==0 )
 			stop();
+		else if ( cam ) {
+			for ( i=0; i<(int)out.count(); i++ ) {
+				if ( out.at(i)->channel.fta ) {
+					i=-1;
+					break;
+				}
+			}
+			if ( i!=-1 )
+				cam->stop();
+		}
 	}
 	recordingState();
 	if ( t )
@@ -757,6 +834,8 @@ int DvbStream::canStartBroadcast( bool &live, ChannelDesc *chan )
 	for ( i=0; i<(int)out.count(); i++ ) {
 		if ( (chan->tp!=out.at(i)->channel.tp) && out.at(i)->hasRec() )
 			return ErrIsRecording;
+		if ( cam && out.at(i)->hasRec() && out.at(i)->channel.fta && chan->fta && out.at(i)->channel.sid!=chan->sid )
+			return ErrCamUsed;
 		if ( out.at(i)->hasLive() && chan->tp!=out.at(i)->channel.tp )
 			live = true;
 	}
@@ -797,7 +876,7 @@ bool DvbStream::startBroadcast( QPtrList<ChannelDesc> *list, Ts2Rtp *rtp )
 		}
 
 		if ( !o ) {
-			o = new DVBout( *list->at(i), adapter, tuner );
+			o = new DVBout( *list->at(i), adapter, tuner, plug );
 			connect( o, SIGNAL(endRecord(DVBout*,RecTimer*,bool)), this, SLOT(recordEnded(DVBout*,RecTimer*,bool)) );
 			connect( o, SIGNAL(playDvb()), this, SLOT(receivePlayDvb()) );
 			connect( o, SIGNAL(shifting(bool)), this, SLOT(receiveShifting(bool)) );
@@ -842,16 +921,24 @@ bool DvbStream::startBroadcast( QPtrList<ChannelDesc> *list, Ts2Rtp *rtp )
 int DvbStream::canStartTimer( bool &live, ChannelDesc *chan )
 {
 	int i, ret=0;
+	DVBout *o;
 
 	for ( i=0; i<(int)out.count(); i++ ) {
-		if ( (chan->tp!=out.at(i)->channel.tp) && out.at(i)->hasRec() )
+		o = out.at(i);
+		if ( (chan->tp!=o->channel.tp) && o->hasRec() )
 			return ErrIsRecording;
-		if ( (out.at(i)->channel.name==chan->name) && out.at(i)->hasRec() )
+		if ( (o->channel.name==chan->name) && o->hasRec() )
 			return ErrIsRecording;
-		if ( out.at(i)->hasLive() ) {
-			if ( chan->tp!=out.at(i)->channel.tp )
+		if ( (chan->tp==o->channel.tp) && o->hasRec() ) {
+			if ( chan->fta && cam && o->channel.fta )
+				return ErrCamUsed;
+		}
+		if ( o->hasLive() ) {
+			if ( chan->tp!=o->channel.tp )
 				live = true;
-			else if ( (out.at(i)->channel.name==chan->name) && (out.at(i)->currentAudioPid()!=0) )
+			else if ( (o->channel.name==chan->name) && (o->currentAudioPid()!=0) )
+				live = true;
+			else if ( cam && chan->fta && o->channel.fta )
 				live = true;
 		}
 	}
@@ -904,7 +991,7 @@ bool DvbStream::startTimer( ChannelDesc *chan, QString path, RecTimer *t )
 	}
 
 	if ( !o ) {
-		o = new DVBout( *chan, adapter, tuner );
+		o = new DVBout( *chan, adapter, tuner, plug );
 		connect( o, SIGNAL(endRecord(DVBout*,RecTimer*,bool)), this, SLOT(recordEnded(DVBout*,RecTimer*,bool)) );
 		connect( o, SIGNAL(playDvb()), this, SLOT(receivePlayDvb()) );
 		connect( o, SIGNAL(shifting(bool)), this, SLOT(receiveShifting(bool)) );
@@ -933,6 +1020,8 @@ bool DvbStream::startTimer( ChannelDesc *chan, QString path, RecTimer *t )
 	}
 
 	fprintf(stderr,"NOUT: %d\n", out.count() );
+	if ( chan->fta && cam && ( ((cam->running() && chan->sid!=cam->serviceId()) || !cam->running()) ) )
+		cam->restart( chan->sid );
 
 	startReading();
 
@@ -953,6 +1042,10 @@ int DvbStream::goLive( ChannelDesc *chan, const QString &pipeName, int &napid )
 			return ErrIsRecording;
 		if ( (chan->tp!=out.at(i)->channel.tp) && out.at(i)->hasBroadcast() )
 			return ErrIsBroadcasting;
+		if ( (chan->tp==out.at(i)->channel.tp) && (out.at(i)->hasBroadcast() || out.at(i)->hasRec())) {
+			if ( chan->fta && cam && cam->running() && (cam->serviceId()!=chan->sid) )
+				return ErrCamUsed;
+		}
 		if ( out.at(i)->channel.name==chan->name )
 			o = out.at(i);
 	}
@@ -973,7 +1066,7 @@ int DvbStream::goLive( ChannelDesc *chan, const QString &pipeName, int &napid )
 	}
 
 	if ( !o ) {
-		o = new DVBout( *chan, adapter, tuner );
+		o = new DVBout( *chan, adapter, tuner, plug );
 		connect( o, SIGNAL(endRecord(DVBout*,RecTimer*,bool)), this, SLOT(recordEnded(DVBout*,RecTimer*,bool)) );
 		connect( o, SIGNAL(playDvb()), this, SLOT(receivePlayDvb()) );
 		connect( o, SIGNAL(shifting(bool)), this, SLOT(receiveShifting(bool)) );
@@ -996,6 +1089,8 @@ int DvbStream::goLive( ChannelDesc *chan, const QString &pipeName, int &napid )
 	o->goLive( pipeName, napid );
 	fprintf(stderr,"NOUT: %d\n", out.count() );
 
+	if ( chan->fta && cam && !cam->running() )
+		cam->restart( chan->sid );
 	startReading();
 
 	return i;
@@ -1024,6 +1119,7 @@ void DvbStream::stopLive( ChannelDesc *chan )
 	int i;
 	DVBout *o;
 	QPtrList<DVBout> p;
+	bool camUsed = false;
 
 	fprintf(stderr,"Asked to stop\n");
 	for ( i=0; i<(int)out.count(); i++ ) {
@@ -1041,6 +1137,14 @@ void DvbStream::stopLive( ChannelDesc *chan )
 		removeOut( p.at(i) );
 	}
 	fprintf(stderr,"Live stopped\n");
+	if ( cam ) {
+		for ( i=0; i<(int)out.count(); i++ ) {
+			if ( out.at(i)->channel.fta )
+				camUsed = true;
+		}
+	}
+	if ( cam && !camUsed )
+		cam->stop();
 	if ( out.count()==0 && chan->tp!=currentTransponder )
 		stop();
 }
@@ -1061,11 +1165,13 @@ void DvbStream::startReading()
 void DvbStream::stop()
 {
 	isRunning = false;
-	if ( !wait(1000) ) {
+	if ( !wait(5000) ) {
 		terminate();
 		wait();
 		fprintf(stderr,"dvbstream::run() terminated\n");
 	}
+	if ( cam )
+		cam->stop();
 	dvbEvents->stop();
 	stopFrontend();
 }
@@ -1101,6 +1207,10 @@ void DvbStream::stopScan()
 DvbStream::~DvbStream()
 {
 	stop();
+	if ( cam ) {
+		cam->stop();
+		delete cam;
+	}
 	delete dvbEvents;
 }
 
